import { AllModules } from '@ag-grid-enterprise/all-modules';
import {
  AgGridEvent,
  ColumnApi,
  GridApi,
  Module,
} from '@ag-grid-enterprise/all-modules/dist/cjs/main';
import './ag-grid-styles.scss';

import 'ag-grid-enterprise';
import 'ag-grid-community/dist/styles/ag-grid.css';
import 'ag-grid-community/dist/styles/ag-theme-material.css';

import '@ag-grid-enterprise/all-modules/dist/styles/ag-theme-material.css';
import { AgGridReact, AgGridReactProps } from '@ag-grid-community/react';
import { ExcelExportModule } from '@ag-grid-enterprise/excel-export';
import { Box, Button, Grid, makeStyles, MenuItem } from '@material-ui/core';
import clsx from 'clsx';
import invariant from 'invariant';
import React, {
  Dispatch,
  FC,
  Reducer,
  useCallback,
  useMemo,
  useReducer,
} from 'react';
import { SplitButton } from './SplitButton';

const useStyles = makeStyles(
  {
    root: {
      flex: 1,
      height: 'calc(100vh - 280px)',
    },
  },
  { name: 'AgTable' }
);

const globalModules: Module[] = [...AllModules, ExcelExportModule];

interface AgGridApi {
  gridApi?: GridApi;
  gridColumnApi?: ColumnApi;
}

interface UseAgGridApi extends AgGridApi {
  onGridReady: Dispatch<AgGridEvent>;
}

/**
 * Utility hook for getting the ag-grid APIs into state
 */
export function useAgGridApi(): UseAgGridApi {
  const [{ gridApi, gridColumnApi }, onGridReady] = useReducer<
    Reducer<AgGridApi, AgGridEvent>
  >((state, params) => {
    return { gridApi: params.api, gridColumnApi: params.columnApi };
  }, {});

  return { gridApi, gridColumnApi, onGridReady };
}

export interface AgTableButttonsProps {
  gridApi?: GridApi;
  gridColumnApi?: ColumnApi;
}

/**
 * Size/Export buttons for AgGrid tables
 */
export const AgTableButtons: FC<AgTableButttonsProps> = props => {
  const { gridApi, gridColumnApi } = props;

  const exportToCSV = useCallback(() => {
    invariant(gridApi, `gridApi not ready`);
    gridApi?.exportDataAsCsv({
      allColumns: true,
      fileName: 'export',
    });
  }, [gridApi]);
  const exportToExcel = useCallback(() => {
    invariant(gridApi, `gridApi not ready`);
    gridApi?.exportDataAsExcel({
      allColumns: true,
      fileName: 'export',
    });
  }, [gridApi]);

  const sizeToFit = useCallback(
    params => {
      invariant(gridApi, `gridApi not ready`);
      gridApi?.sizeColumnsToFit();
    },
    [gridApi]
  );

  const autoSizeAll = useCallback(
    skipHeader => {
      invariant(gridColumnApi, `gridColumnApi not ready`);
      const allColumnIds: string[] = [];
      gridColumnApi?.getAllColumns()?.forEach((column: { getColId: () => string; }) => {
        allColumnIds.push(column.getColId());
      });
      gridColumnApi?.autoSizeColumns(allColumnIds, skipHeader);
    },
    [gridColumnApi]
  );
  const autoSizeAllWithHeader = useCallback(() => autoSizeAll(false), [
    autoSizeAll,
  ]);
  const autoSizeAllWithoutHeader = useCallback(() => autoSizeAll(false), [
    autoSizeAll,
  ]);

  return (
    <Box mb={1}>
      <Grid container direction="row" spacing={1} justify="flex-end">
        <Grid item>
          <Button variant="outlined" size="small" onClick={sizeToFit}>
            Size to Fit
          </Button>
        </Grid>
        <Grid item>
          <SplitButton
            popupName="auto-size"
            variant="outlined"
            size="small"
            label="Auto-Size"
          >
            <MenuItem onClick={autoSizeAllWithHeader}>All</MenuItem>
            <MenuItem onClick={autoSizeAllWithoutHeader}>Skip Header</MenuItem>
          </SplitButton>
        </Grid>
        <Grid item>
          <SplitButton
            popupName="export"
            variant="outlined"
            size="small"
            label="Export"
          >
            <MenuItem onClick={exportToExcel}>Excel</MenuItem>
            <MenuItem onClick={exportToCSV}>CSV</MenuItem>
          </SplitButton>
        </Grid>
      </Grid>
    </Box>
  );
};

type AgTableProps = AgGridReactProps;

/**
 * AdGrid with tweaks
 */
export const AgTable: FC<AgTableProps> = props => {
  const classes = useStyles(props);
  const { modules: modulesProp } = props;
  const modules = useMemo<Module[]>(
    () => (modulesProp ? globalModules.concat(modulesProp) : globalModules),
    [modulesProp]
  );

  return (
    <div className={clsx('ag-theme-material', classes.root)}>
      <AgGridReact {...props} modules={modules} />
    </div>
  );
};
