import React from 'react'
import PropTypes from 'prop-types'
import { makeStyles } from '@material-ui/core/styles'
import TreeView from '@material-ui/lab/TreeView'
import TreeItem from '@material-ui/lab/TreeItem'
import Typography from '@material-ui/core/Typography'
import {
  ChevronDownIcon,
  ChevronRightIcon,
  ViewDashboardIcon,
  AdminALogo32Icon,
  MessageIcon,
  ContentFeedIcon,
  RawSourceIcon,
  BugIcon,
  OfficeFormsLogo16Icon,
  CloudUploadIcon
} from '@fluentui/react-icons'
import { Link } from 'react-router-dom'

const useTreeItemStyles = makeStyles((theme) => ({
  root: {
    color: theme.palette.text.secondary,
    '&:hover > $content': {
      backgroundColor: theme.palette.action.hover
    },
    '&:focus > $content, &$selected > $content': {
      backgroundColor: `var(--tree-view-bg-color, ${theme.palette.grey[300]})`,
      color: 'var(--tree-view-color)'
    },
    '&:focus > $marker, &$selected > $marker': {
      backgroundColor: `var(--tree-view-bg-color, ${theme.palette.grey[300]})`,
      color: 'var(--tree-view-color)'
    },
    '&:focus > $content $label, &:hover > $content $label, &$selected > $content $label': {
      backgroundColor: 'transparent'
    }
  },
  content: {
    color: theme.palette.text.secondary,
    paddingRight: theme.spacing(1),
    paddingLeft: theme.spacing(1),
    fontWeight: theme.typography.fontWeightMedium,
    '$expanded > &': {
      fontWeight: theme.typography.fontWeightRegular
    }
  },
  group: {
    marginLeft: 0,
    '& $content': {
      paddingLeft: theme.spacing(2)
    }
  },
  expanded: {},
  selected: {},
  label: {
    fontWeight: 'inherit',
    color: 'inherit'
  },
  labelRoot: {
    display: 'flex',
    alignItems: 'center',
    padding: theme.spacing(1.5, 0)
  },
  labelIcon: {
    marginRight: theme.spacing(2.5),
    fontSize: 16,
    color: theme.palette.secondary.main
  },
  labelText: {
    fontWeight: 'inherit',
    fontSize: 14,
    flexGrow: 1
  }
}))

const StyledTreeItem = (props) => {
  const classes = useTreeItemStyles()
  const {
    labelText,
    labelIcon: LabelIcon,
    labelInfo,
    color,
    bgColor,
    to,
    children,
    ...other
  } = props

  return (
    <Link to={to} style={{ textDecoration: 'none' }}>
      <TreeItem
        label={
          <div className={classes.labelRoot}>
            {LabelIcon && (
              <LabelIcon color='inherit' className={classes.labelIcon} />
            )}
            <Typography variant='body2' className={classes.labelText}>
              {labelText}
            </Typography>
            <Typography variant='caption' color='inherit'>
              {labelInfo}
            </Typography>
          </div>
        }
        style={{
          '--tree-view-color': color,
          '--tree-view-bg-color': bgColor
        }}
        classes={{
          root: classes.root,
          content: classes.content,
          expanded: classes.expanded,
          selected: classes.selected,
          group: classes.group,
          label: classes.label
        }}
        {...other}
        children={children}
      />
    </Link>
  )
}

StyledTreeItem.propTypes = {
  bgColor: PropTypes.string,
  color: PropTypes.string,
  labelIcon: PropTypes.elementType,
  labelInfo: PropTypes.string,
  labelText: PropTypes.string.isRequired
}

const useStyles = makeStyles({
  root: {
    height: 264,
    flexGrow: 1,
    maxWidth: 280
  }
})

export const FluentSideNav = (props) => {
  const classes = useStyles()
  const { data } = props

  const renderedArray = data.map((nodes) => {
    const icons = {
      ViewDashboardIcon,
      AdminALogo32Icon,
      MessageIcon,
      ContentFeedIcon,
      RawSourceIcon,
      BugIcon,
      OfficeFormsLogo16Icon,
      CloudUploadIcon
    }
    return (
      <StyledTreeItem
        key={nodes.nodeId}
        nodeId={nodes.nodeId}
        labelText={nodes.labelText}
        labelIcon={
          typeof nodes.labelIcon === 'string'
            ? icons[nodes.labelIcon]
            : nodes.labelIcon
        }
        to={nodes.to !== '' && nodes.to}
        children={
          nodes.children &&
          nodes.children.map((nodes) => {
            return (
              <StyledTreeItem
                key={nodes.nodeId}
                nodeId={nodes.nodeId}
                labelText={nodes.labelText}
                labelIcon={
                  typeof nodes.labelIcon === 'string'
                    ? icons[nodes.labelIcon]
                    : nodes.labelIcon
                }
                to={nodes.to !== '' && nodes.to}
              />
            )
          })
        }
      />
    )
  })

  return (
    <TreeView
      className={classes.root}
      defaultCollapseIcon={<ChevronDownIcon />}
      defaultExpandIcon={<ChevronRightIcon />}
      defaultEndIcon={<div style={{ width: 24 }} />}
      // defaultExpanded={[{ expanded }]}
    >
      {renderedArray}
    </TreeView>
  )
}
