import React, { useState } from 'react'
import PropTypes from 'prop-types'
import {
  makeStyles,
  createStyles,
  List,
  ListItemIcon,
  ListItemText,
  Collapse
} from '@material-ui/core'
import { ChevronDownIcon, ChevronUpIcon } from '@fluentui/react-icons'
import { AppMenuItemComponent } from './AppMenuItemComponent'
import { FluentIcon } from '../../../utilities/FluentIcon'
import { redirectToLinkOnClick } from '../../../utilities/menus/redirectToLink'

// React runtime PropTypes
export const AppMenuItemPropTypes = {
  id: PropTypes.string,
  label: PropTypes.string.isRequired,
  link: PropTypes.string,
  icon: PropTypes.elementType,
  iconName: PropTypes.string,
  items: PropTypes.array,
  indent: PropTypes.string,
  redirectInfo: PropTypes.string,
  params: PropTypes.object,
  menuItemId: PropTypes.string,
  menuStructureId: PropTypes.number,
  parentId: PropTypes.number,
  productId: PropTypes.number,
  isActive: PropTypes.bool,
  roleIds: PropTypes.array,
  childrenIcons: PropTypes.bool,
  isChild: PropTypes.bool,
  childIndex: PropTypes.number
}

const useStyles = makeStyles((theme) =>
  createStyles({
    menuItem: {
      '&.active': {
        background: 'rgba(0, 0, 0, 0.08)',
        '& .MuiListItemIcon-root': { marginLeft: theme.spacing(2) }
      }
    },
    menuItemIcon: {
      minWidth: theme.spacing(5)
    },
    indent: {},
    icon: {
      width: theme.spacing(3),
      height: theme.spacing(3),
      color: theme.palette.secondary.main
    },
    expand: {
      marginRight: theme.spacing(1)
    },
    list: {
      '& .MuiListItemText-root': {
        paddingLeft: (props) =>
          `calc(16px + ${
            16 * (props.childIndex !== undefined ? props.childIndex : 0)
          }px)`
      }
    }
  })
)

export const AppMenuItem = (props) => {
  const {
    childrenIcons = true,
    isChild = false,
    childIndex,
    label,
    link,
    icon,
    indent,
    items = [],
    redirectInfo,
    ...other
  } = props
  const classes = useStyles(props)
  const isExpandable = items?.length > 0
  const [open, setOpen] = useState(false)

  function handleClick() {
    if (redirectInfo) {
      redirectToLinkOnClick(redirectInfo) // TODO for demo purposes
    }
    setOpen(!open)
  }

  const MenuItemRoot = (
    <AppMenuItemComponent
      className={indent ? classes.menuItem : classes.indent}
      link={link}
      onClick={handleClick}
      aria-expanded={isExpandable ? open : null}
      params={other}
      isChild={isChild}
    >
      {/* Display an icon if any */}
      {!!icon && ((isChild && childrenIcons) || !isChild) && (
        <ListItemIcon className={classes.menuItemIcon}>
          <div className={classes.icon}>
            <FluentIcon component={icon} />
          </div>
        </ListItemIcon>
      )}
      <ListItemText primary={label} inset={!icon} />
      {/* Display the expand menu if the item has children */}
      {isExpandable &&
        (open ? (
          <div className={classes.expand}>
            <FluentIcon component={ChevronUpIcon} />
          </div>
        ) : (
          <div className={classes.expand}>
            <FluentIcon component={ChevronDownIcon} />
          </div>
        ))}
    </AppMenuItemComponent>
  )

  const MenuItemChildren = isExpandable ? (
    <Collapse in={open} timeout='auto' unmountOnExit>
      {/* <Divider aria-hidden={true} /> */}
      <List className={classes.list}>
        {items.map((item, index) => (
          <AppMenuItem
            {...item}
            childrenIcons={childrenIcons}
            isChild
            key={index}
            childIndex={index}
          />
        ))}
      </List>
    </Collapse>
  ) : null

  return (
    <li>
      {MenuItemRoot}
      {MenuItemChildren}
    </li>
  )
}
