import React, { forwardRef, Fragment } from 'react'
import { Grid, Box, Typography } from '@material-ui/core'
import { MuiVegaLite } from '@material-vega/material-ui'
import WidgetCard from '../WidgetCard'
import FluentStatsSnippet from '../FluentComponents/atoms/FluentStatsSnippet'
import ChartSummaryToggle from '../FluentComponents/ChartSummaryToggle'
import { convertISODate } from '../utils/ConvertISODate'

export const layout = {
  w: { default: 2, min: 0, max: 4 },
  h: { default: 2, min: 0, max: 4 }
}

const DonutChartWidget = (props, ref) => {
  const { children, params, setParam, ...other } = props
  const { mockData, apiUrl, theme, dateRange } = params

  let body = (
    <Fragment>
      {dateRange && (
        <Typography gutterBottom variant='caption' component='h3'>
          From: {convertISODate(dateRange.from)} to:{' '}
          {convertISODate(dateRange.to)}
        </Typography>
      )}
      <Grid container>
        {mockData?.stats?.map((stat) => (
          <Grid item xs={3} key={stat.label}>
            <FluentStatsSnippet
              label={stat.label}
              amount={stat.currentValue}
              prevAmount={stat.previousValue}
            />
          </Grid>
        ))}
      </Grid>
      <Box clone flex={1} mt={1} overflow='hidden'>
        <MuiVegaLite
          variant='donutChart'
          style={{
            overflow: 'auto',
            display: 'flex',
            alignItems: 'center',
            justifyContent: 'center',
            padding: 0
          }}
          spec={{
            $schema: 'https://vega.github.io/schema/vega/v5.json',
            width: 200,
            height: 200,
            data: [
              {
                name: 'table',
                values: mockData?.data,
                transform: [
                  {
                    type: 'pie',
                    field: 'value'
                  }
                ]
              },
              {
                name: 'used',
                source: 'table',
                transform: [
                  {
                    type: 'formula',
                    expr: "datum.id === 'Other' ? 'Other' : 'Main'",
                    as: 'id'
                  },
                  {
                    type: 'aggregate',
                    fields: ['field'],
                    groupby: ['id'],
                    ops: ['sum'],
                    as: ['sum']
                  },
                  {
                    type: 'pivot',
                    field: 'label',
                    value: 'sum'
                  }
                ]
              }
            ],
            scales: [
              {
                name: 'color',
                type: 'ordinal',
                domain: { data: 'table', field: 'label' },
                range: { scheme: 'greys' }
              }
            ],
            marks: [
              {
                type: 'arc',
                from: { data: 'table' },
                encode: {
                  enter: {
                    fill: { scale: 'color', field: 'label' },
                    x: { signal: 'width / 2' },
                    y: { signal: 'height / 2' },
                    padAngle: { value: 0.01 }
                  },
                  update: {
                    startAngle: { field: 'startAngle' },
                    endAngle: { field: 'endAngle' },
                    innerRadius: { signal: 'width / 2 - 10' },
                    outerRadius: { signal: 'width / 2' }
                  }
                }
              },
              {
                type: 'text',
                from: { data: 'used' },
                style: ['percent'],
                encode: {
                  enter: {
                    x: { signal: 'width / 2' },
                    y: { signal: 'height / 2' },
                    align: { value: 'center' },
                    baseline: { value: 'middle' },
                    fontSize: { value: 24 }
                  }
                }
              }
            ],
            legends: [
              {
                fill: 'color',
                direction: 'vertical',
                encode: {
                  labels: {
                    name: 'legendLabel',
                    enter: { text: { field: 'label' } }
                  }
                },
                orient: 'right',
                labelColor: theme === 'dark' ? 'white' : 'rgba(0, 0, 0, 0.87)'
              }
            ]
          }}
        />
      </Box>
      {children}
    </Fragment>
  )

  return (
    <WidgetCard
      theme={theme}
      ref={ref}
      headerActions={
        <ChartSummaryToggle
          value={params.mode}
          onChange={(value) => setParam('mode', value)}
        />
      }
      {...other}
      title={mockData?.title}
    >
      {mockData ? (
        body
      ) : (
        <Typography variant='subtitle2'>
          Could not retrieve widget data. Please contact support.
        </Typography>
      )}
    </WidgetCard>
  )
}

export default forwardRef(DonutChartWidget)
