import React, { useRef, useState, useEffect } from 'react'
import './Map.css'
import MapContext from './MapContext'
import * as ol from 'ol'
import {
  DragRotateAndZoom,
  defaults as defaultInteractions,
  Select as OlSelect
} from 'ol/interaction'
import { Style as OlStyle, Icon as OlIcon } from 'ol/style'

import { click as olClick } from 'ol/events/condition'
import GeoLocation from '../geoLocation/GeoLocation'
import { fromLonLat } from 'ol/proj'
import { usePrevious } from '../../utilities/usePrevious'

export const Map = ({
  children,
  zoom,
  center,
  projection,
  addInteractions,
  sendSelected,
  sendType,
  SelectedPin,
  changeLat,
  changeLong,
  setPosition,
  width,
  height,
  addressBar
}) => {
  const mapRef = useRef()
  const [map, setMap] = useState(null)
  const [selected, setSelected] = useState()
  const [markingType, setMarkingType] = useState()
  const [count, setCount] = useState(0)
  const [mapClicked, setMapclicked] = useState(false)

  const prevCount = usePrevious(count)

  // on component mount
  useEffect(() => {
    const options = {
      view: new ol.View({
        center: fromLonLat(center),
        zoom,
        projection
      }),
      interactions: defaultInteractions().extend([new DragRotateAndZoom()]),
      layers: [],
      controls: [],
      overlays: []
    }

    const mapObject = new ol.Map(options)
    mapObject.setTarget(mapRef.current)
    setMap(mapObject)

    const pointSelect = new OlSelect({
      condition: (e) => {
        return olClick(e)
      },
      filter: (f) => {
        if (f.getGeometry().getType() === 'Point') {
          return true
        }
      },
      multi: true,
      hitTolerance: 10,
      style: new OlStyle({
        image: new OlIcon({
          anchor: [0.5, 48],
          anchorXUnits: 'fraction',
          anchorYUnits: 'pixels',
          src: SelectedPin
        })
      })
    })
    // pointSelect.on('select', e => console.log('E: ', e.selected));
    mapObject.addInteraction(pointSelect)
    mapObject.on('click', (e) => {
      mapObject.forEachFeatureAtPixel(e.pixel, (a) => console.log('A: ', a.id_))
    })

    return () => mapObject.setTarget(undefined)
  }, [])

  useEffect(() => {
    sendSelected(selected)
  }, [sendSelected, selected])

  useEffect(() => {
    sendType(markingType)
  }, [sendType, markingType])

  const postSelected = (id, type) => {
    setSelected(id)
    setMarkingType(type)
  }
  // console.log('ID, TYPE', selected, markingType);

  // zoom change handler
  useEffect(() => {
    if (!map) return

    map.getView().setZoom(zoom)
    console.log('run zoom')
  }, [zoom])

  // click event handler
  useEffect(() => {
    if (!map) return

    map.on('click', (e) => {
      setMapclicked(true)
      map.forEachFeatureAtPixel(
        e.pixel,
        // a => setSelected(a.values_.geometry.flatCoordinates[2])
        (a) =>
          postSelected(
            a.values_.geometry.flatCoordinates[2],
            a.values_.geometry.flatCoordinates[3]
          )
      )
    })
  }, [map])

  // center change handler - Caused issue of recentering the map on each map selection when passed.
  useEffect(() => {
    if (!map) return
    console.log('Run useEffect center', center, zoom)

    console.log('PREV', prevCount)
    setCount(zoom)
    if (zoom === prevCount && !mapClicked) {
      map.getView().setCenter(center)
    }
  }, [center])

  useEffect(() => {
    if (!map) return

    if (!mapClicked) {
      map.getView().setCenter(center)
    }
  }, [map])

  useEffect(() => {
    if (!map) return
    map.getview().getprojection(projection)
  }, [projection])

  return (
    <MapContext.Provider value={{ map }}>
      {addressBar && (
        <div style={{ marginLeft: 35, marginTop: -6 }}>
          <GeoLocation
            // sendSelected={setCenter}
            center={center}
            changeLat={changeLat}
            changeLong={changeLong}
            setPosition={setPosition}
          />
        </div>
      )}

      <div ref={mapRef} style={{ height: height || '85vh', width: width }}>
        {children}
      </div>
    </MapContext.Provider>
  )
}
